@@ -3,6 +3,7 @@ import gc
 import inspect
 import json
 import logging
+import os.path
 import pdb
 import re
 import sys
@@ -11,7 +12,7 @@ import time
 from collections.abc import Awaitable, Callable
 from datetime import datetime
 from pathlib import Path
-from typing import Any, Generic, Literal, TypeVar
+from typing import Any, Generic, Literal, TypeVar, Optional
 from urllib.parse import urlparse
 
 from dotenv import load_dotenv
@@ -53,12 +54,12 @@ from browser_use.agent.views import (
     StepMetadata,
 )
 from pydantic import BaseModel, ConfigDict, Field, ValidationError, create_model, model_validator
-from browser_use.browser import BrowserProfile, BrowserSession
+from browser_use import Browser, BrowserProfile, BrowserSession
 from browser_use.browser.session import DEFAULT_BROWSER_PROFILE
 from browser_use.browser.views import BrowserStateSummary
 from browser_use.config import CONFIG
-from browser_use.controller.registry.views import ActionModel
-from browser_use.controller.service import Controller
+from browser_use.tools.registry.views import ActionModel
+from browser_use.tools.service import Controller, Tools
 from browser_use.dom.views import DOMInteractedElement
 from browser_use.filesystem.file_system import FileSystem
 from browser_use.observability import observe, observe_debug
@@ -74,51 +75,11 @@ from browser_use.utils import (
 )
 
 from browser_use.agent.service import Agent, AgentHookFunc
-from vibe_surf.controller.file_system import CustomFileSystem
+from vibe_surf.tools.file_system import CustomFileSystem
 
 Context = TypeVar('Context')
 
 
-def log_response(response: AgentOutput, registry=None, logger=None) -> None:
-    """Utility function to log the model's response."""
-
-    # Use module logger if no logger provided
-    if logger is None:
-        logger = logging.getLogger(__name__)
-
-    # Only log thinking if it's present
-    if response.current_state.thinking:
-        logger.info(f'💡 Thinking:\n{response.current_state.thinking}')
-
-    # Only log evaluation if it's not empty
-    eval_goal = response.current_state.evaluation_previous_goal
-    if eval_goal:
-        if 'success' in eval_goal.lower():
-            emoji = '👍'
-            # Green color for success
-            logger.info(f'  \033[32m{emoji} Eval: {eval_goal}\033[0m')
-        elif 'failure' in eval_goal.lower():
-            emoji = '⚠️'
-            # Red color for failure
-            logger.info(f'  \033[31m{emoji} Eval: {eval_goal}\033[0m')
-        else:
-            emoji = '❔'
-            # No color for unknown/neutral
-            logger.info(f'  {emoji} Eval: {eval_goal}')
-
-    # Always log memory if present
-    if response.current_state.memory:
-        logger.debug(f'🧠 Memory: {response.current_state.memory}')
-
-    # Only log next goal if it's not empty
-    next_goal = response.current_state.next_goal
-    if next_goal:
-        # Blue color for next goal
-        logger.info(f'  \033[34m🎯 Next goal: {next_goal}\033[0m')
-    else:
-        logger.info('')  # Add empty line for spacing
-
-
 class BrowserUseAgent(Agent):
     @time_execution_sync('--init')
     def __init__(
@@ -128,7 +89,9 @@ class BrowserUseAgent(Agent):
             # Optional parameters
             browser_profile: BrowserProfile | None = None,
             browser_session: BrowserSession | None = None,
-            controller: Controller[Context] | None = None,
+            browser: Browser | None = None,  # Alias for browser_session
+            tools: Tools[Context] | None = None,
+            controller: Tools[Context] | None = None,  # Alias for tools
             # Initial agent run parameters
             sensitive_data: dict[str, str | dict[str, str]] | None = None,
             initial_actions: list[dict[str, dict[str, Any]]] | None = None,
@@ -147,14 +110,11 @@ class BrowserUseAgent(Agent):
             # Agent settings
             output_model_schema: type[AgentStructuredOutput] | None = None,
             use_vision: bool = True,
-            use_vision_for_planner: bool = False,  # Deprecated
             save_conversation_path: str | Path | None = None,
             save_conversation_path_encoding: str | None = 'utf-8',
             max_failures: int = 3,
-            retry_delay: int = 10,
             override_system_message: str | None = None,
             extend_system_message: str | None = None,
-            validate_output: bool = False,
             generate_gif: bool | str = False,
             available_file_paths: list[str] | None = None,
             include_attributes: list[str] | None = None,
@@ -163,12 +123,7 @@ class BrowserUseAgent(Agent):
             flash_mode: bool = False,
             max_history_items: int | None = None,
             page_extraction_llm: BaseChatModel | None = None,
-            planner_llm: BaseChatModel | None = None,  # Deprecated
-            planner_interval: int = 1,  # Deprecated
-            is_planner_reasoning: bool = False,  # Deprecated
-            extend_planner_system_message: str | None = None,  # Deprecated
             injected_agent_state: AgentState | None = None,
-            context: Context | None = None,
             source: str | None = None,
             file_system_path: str | None = None,
             task_id: str | None = None,
@@ -179,38 +134,13 @@ class BrowserUseAgent(Agent):
             vision_detail_level: Literal['auto', 'low', 'high'] = 'auto',
             llm_timeout: int = 90,
             step_timeout: int = 120,
-            preload: bool = True,
+            directly_open_url: bool = False,
             include_recent_events: bool = False,
             allow_parallel_action_types: list[str] = ["extract_structured_data", "extract_content_from_file"],
+            _url_shortening_limit: int = 25,
+            token_cost_service: Optional[TokenCost] = None,
             **kwargs,
     ):
-        if not isinstance(llm, BaseChatModel):
-            raise ValueError('invalid llm, must be from browser_use.llm')
-        # Check for deprecated planner parameters
-        planner_params = [
-            planner_llm,
-            use_vision_for_planner,
-            is_planner_reasoning,
-            extend_planner_system_message,
-        ]
-        if any(param is not None and param is not False for param in planner_params) or planner_interval != 1:
-            self.logger.warning(
-                '⚠️ Planner functionality has been removed in browser-use v0.3.3+. '
-                'The planner_llm, use_vision_for_planner, planner_interval, is_planner_reasoning, '
-                'and extend_planner_system_message parameters are deprecated and will be ignored. '
-                'Please remove these parameters from your Agent() initialization.'
-            )
-
-        # Check for deprecated memory parameters
-        if kwargs.get('enable_memory', False) or kwargs.get('memory_config') is not None:
-            self.logger.warning(
-                'Memory support has been removed as of version 0.3.2. '
-                'The agent context for memory is significantly improved and no longer requires the old memory system. '
-                "Please remove the 'enable_memory' and 'memory_config' parameters."
-            )
-            kwargs['enable_memory'] = False
-            kwargs['memory_config'] = None
-
         if page_extraction_llm is None:
             page_extraction_llm = llm
         if available_file_paths is None:
@@ -220,40 +150,51 @@ class BrowserUseAgent(Agent):
         self.task_id: str = self.id
         self.session_id: str = uuid7str()
         self.allow_parallel_action_types = allow_parallel_action_types
+        self._url_shortening_limit = _url_shortening_limit
+
+        browser_profile = browser_profile or DEFAULT_BROWSER_PROFILE
+
+        # Handle browser vs browser_session parameter (browser takes precedence)
+        if browser and browser_session:
+            raise ValueError(
+                'Cannot specify both "browser" and "browser_session" parameters. Use "browser" for the cleaner API.')
+        browser_session = browser or browser_session
+
+        self.browser_session = browser_session or BrowserSession(
+            browser_profile=browser_profile,
+            id=uuid7str()[:-4] + self.id[-4:],  # re-use the same 4-char suffix so they show up together in logs
+        )
 
         # Initialize available file paths as direct attribute
         self.available_file_paths = available_file_paths
 
-        # Temporary logger for initialization (will be replaced by property)
-        self._logger = None
-
         # Core components
         self.task = task
         self.llm = llm
-        self.preload = preload
+        self.directly_open_url = directly_open_url
         self.include_recent_events = include_recent_events
-        self.controller = (
-            controller if controller is not None else Controller(display_files_in_done_text=display_files_in_done_text)
-        )
+        if tools is not None:
+            self.tools = tools
+        elif controller is not None:
+            self.tools = controller
+        else:
+            self.tools = Tools(display_files_in_done_text=display_files_in_done_text)
 
         # Structured output
         self.output_model_schema = output_model_schema
         if self.output_model_schema is not None:
-            self.controller.use_structured_output_action(self.output_model_schema)
+            self.tools.use_structured_output_action(self.output_model_schema)
 
         self.sensitive_data = sensitive_data
 
         self.settings = AgentSettings(
             use_vision=use_vision,
             vision_detail_level=vision_detail_level,
-            use_vision_for_planner=False,  # Always False now (deprecated)
             save_conversation_path=save_conversation_path,
             save_conversation_path_encoding=save_conversation_path_encoding,
             max_failures=max_failures,
-            retry_delay=retry_delay,
             override_system_message=override_system_message,
             extend_system_message=extend_system_message,
-            validate_output=validate_output,
             generate_gif=generate_gif,
             include_attributes=include_attributes,
             max_actions_per_step=max_actions_per_step,
@@ -261,10 +202,6 @@ class BrowserUseAgent(Agent):
             flash_mode=flash_mode,
             max_history_items=max_history_items,
             page_extraction_llm=page_extraction_llm,
-            planner_llm=None,  # Always None now (deprecated)
-            planner_interval=1,  # Always 1 now (deprecated)
-            is_planner_reasoning=False,  # Always False now (deprecated)
-            extend_planner_system_message=None,  # Always None now (deprecated)
             calculate_cost=calculate_cost,
             include_tool_call_examples=include_tool_call_examples,
             llm_timeout=llm_timeout,
@@ -272,10 +209,12 @@ class BrowserUseAgent(Agent):
         )
 
         # Token cost service
-        self.token_cost_service = TokenCost(include_cost=calculate_cost)
+        if token_cost_service is None:
+            self.token_cost_service = TokenCost(include_cost=calculate_cost)
+        else:
+            self.token_cost_service = token_cost_service
         self.token_cost_service.register_llm(llm)
         self.token_cost_service.register_llm(page_extraction_llm)
-        # Note: No longer registering planner_llm (deprecated)
 
         # Initialize state
         self.state = injected_agent_state or AgentState()
@@ -297,8 +236,19 @@ class BrowserUseAgent(Agent):
         # Action setup
         self._setup_action_models()
         self._set_browser_use_version_and_source(source)
-        self.initial_actions = self._convert_initial_actions(initial_actions) if initial_actions else None
 
+        initial_url = None
+
+        # only load url if no initial actions are provided
+        if self.directly_open_url and not self.state.follow_up_task and not initial_actions:
+            initial_url = self._extract_url_from_task(self.task)
+            if initial_url:
+                self.logger.info(f'🔗 Found URL in task: {initial_url}, adding as initial action...')
+                initial_actions = [{'go_to_url': {'url': initial_url, 'new_tab': False}}]
+
+        self.initial_url = initial_url
+
+        self.initial_actions = self._convert_initial_actions(initial_actions) if initial_actions else None
         # Verify we can connect to the model
         self._verify_and_setup_llm()
 
@@ -308,40 +258,44 @@ class BrowserUseAgent(Agent):
             self.logger.warning(
                 '⚠️ DeepSeek models do not support use_vision=True yet. Setting use_vision=False for now...')
             self.settings.use_vision = False
-        # Note: No longer checking planner_llm for DeepSeek (deprecated)
+
+        if 'kimi-k2' in self.llm.model.lower():
+            self.logger.warning(
+                '⚠️ Kimi-k2 models do not support use_vision=True yet. Setting use_vision=False for now...')
+            self.settings.use_vision = False
+
+        if "qwen" in self.llm.model.lower() and "vl" not in self.llm.model.lower():
+            self.logger.warning("⚠️ Qwen without VL doesn't support vision. Ignore Vision input.")
+            self.settings.use_vision = False
 
         # Handle users trying to use use_vision=True with XAI models
         if 'grok' in self.llm.model.lower():
             self.logger.warning('⚠️ XAI models do not support use_vision=True yet. Setting use_vision=False for now...')
             self.settings.use_vision = False
-        # Note: No longer checking planner_llm for XAI models (deprecated)
 
         self.logger.info(f'🧠 Starting a browser-use version {self.version} with model={self.llm.model}')
-        self.logger.debug(
+        self.logger.info(
             f'{" +vision" if self.settings.use_vision else ""}'
             f' extraction_model={self.settings.page_extraction_llm.model if self.settings.page_extraction_llm else "Unknown"}'
-            # Note: No longer logging planner_model (deprecated)
             f'{" +file_system" if self.file_system else ""}'
         )
 
         # Initialize available actions for system prompt (only non-filtered actions)
         # These will be used for the system prompt to maintain caching
-        self.unfiltered_actions = self.controller.registry.get_prompt_description()
+        self.unfiltered_actions = self.tools.registry.get_prompt_description()
 
         # Initialize message manager with state
         # Initial system prompt with all actions - will be updated during each step
-        system_message = SystemPrompt(
+        self._message_manager = MessageManager(
+            task=task,
+            system_message=SystemPrompt(
                 action_description=self.unfiltered_actions,
                 max_actions_per_step=self.settings.max_actions_per_step,
                 override_system_message=override_system_message,
                 extend_system_message=extend_system_message,
                 use_thinking=self.settings.use_thinking,
                 flash_mode=self.settings.flash_mode,
-            ).get_system_message()
-        self.logger.debug(system_message)
-        self._message_manager = MessageManager(
-            task=task,
-            system_message=system_message,
+            ).get_system_message(),
             file_system=self.file_system,
             state=self.state.message_manager_state,
             use_thinking=self.settings.use_thinking,
@@ -354,13 +308,6 @@ class BrowserUseAgent(Agent):
             include_recent_events=self.include_recent_events,
         )
 
-        browser_profile = browser_profile or DEFAULT_BROWSER_PROFILE
-
-        self.browser_session = browser_session or BrowserSession(
-            browser_profile=browser_profile,
-            id=uuid7str()[:-4] + self.id[-4:],  # re-use the same 4-char suffix so they show up together in logs
-        )
-
         if self.sensitive_data:
             # Check if sensitive_data has domain-specific credentials
             has_domain_specific_credentials = any(isinstance(v, dict) for v in self.sensitive_data.values())
@@ -430,9 +377,6 @@ class BrowserUseAgent(Agent):
         self.register_done_callback = register_done_callback
         self.register_external_agent_status_raise_error_callback = register_external_agent_status_raise_error_callback
 
-        # Context
-        self.context: Context | None = context
-
         # Telemetry
         self.telemetry = ProductTelemetry()
 
@@ -539,6 +483,13 @@ class BrowserUseAgent(Agent):
         # Increment step counter after step is fully completed
         self.state.n_steps += 1
 
+    def add_new_task(self, new_task: str) -> None:
+        """Add a new task to the agent, keeping the same task_id as tasks are continuous"""
+        # Simply delegate to message manager - no need for new task_id or events
+        # The task continues with new instructions, it doesn't end and start a new one
+        self.task = new_task
+        self._message_manager.add_new_task(new_task)
+
     @observe(name='agent.run', metadata={'task': '{{task}}', 'debug': '{{debug}}'})
     @time_execution_async('--run')
     async def run(
@@ -574,7 +525,7 @@ class BrowserUseAgent(Agent):
         signal_handler.register()
 
         try:
-            self._log_agent_run()
+            await self._log_agent_run()
 
             self.logger.debug(
                 f'🔧 Agent setup: Task ID {self.task_id[-4:]}, Session ID {self.session_id[-4:]}, Browser Session ID {self.browser_session.id[-4:] if self.browser_session else "None"}'
@@ -586,39 +537,6 @@ class BrowserUseAgent(Agent):
 
             self.logger.debug('🔧 Browser session started with watchdogs attached')
 
-            # Check if task contains a URL and add it as an initial action (only if preload is enabled)
-            if self.preload:
-                initial_url = self._extract_url_from_task(self.task)
-                if initial_url:
-                    self.logger.info(f'🔗 Found URL in task: {initial_url}, adding as initial action...')
-
-                    # Create a go_to_url action for the initial URL
-                    go_to_url_action = {
-                        'go_to_url': {
-                            'url': initial_url,
-                            'new_tab': False,  # Navigate in current tab
-                        }
-                    }
-
-                    # Add to initial_actions or create new list if none exist
-                    if self.initial_actions:
-                        # Convert back to dict format, prepend URL navigation, then convert back
-                        initial_actions_dicts = []
-                        for action in self.initial_actions:
-                            action_data = action.model_dump(exclude_unset=True)
-                            initial_actions_dicts.append(action_data)
-
-                        # Prepend the go_to_url action
-                        initial_actions_dicts = [go_to_url_action] + initial_actions_dicts
-
-                        # Convert back to ActionModel instances
-                        self.initial_actions = self._convert_initial_actions(initial_actions_dicts)
-                    else:
-                        # Create new initial_actions with just the go_to_url
-                        self.initial_actions = self._convert_initial_actions([go_to_url_action])
-
-                    self.logger.debug(f'✅ Added navigation to {initial_url} as initial action')
-
             # Execute initial actions if provided
             if self.initial_actions:
                 self.logger.debug(f'⚡ Executing {len(self.initial_actions)} initial actions...')
@@ -631,11 +549,13 @@ class BrowserUseAgent(Agent):
                 # Replace the polling with clean pause-wait
                 if self.state.paused:
                     self.logger.debug(f'⏸️ Step {step}: Agent paused, waiting to resume...')
-                    await self.wait_until_resumed()
+                    await self._external_pause_event.wait()
                     signal_handler.reset()
 
                 # Check if we should stop due to too many failures
-                if self.state.consecutive_failures >= self.settings.max_failures:
+                if (self.state.consecutive_failures) >= self.settings.max_failures + int(
+                        self.settings.final_response_after_failure
+                ):
                     self.logger.error(f'❌ Stopping due to {self.settings.max_failures} consecutive failures')
                     agent_run_error = f'Stopped due to {self.settings.max_failures} consecutive failures'
                     break
@@ -646,12 +566,6 @@ class BrowserUseAgent(Agent):
                     agent_run_error = 'Agent stopped programmatically'
                     break
 
-                while self.state.paused:
-                    await asyncio.sleep(0.2)  # Small delay to prevent CPU spinning
-                    if self.state.stopped:  # Allow stopping while paused
-                        agent_run_error = 'Agent stopped programmatically while paused'
-                        break
-
                 if on_step_start is not None:
                     await on_step_start(self)
 
@@ -734,6 +648,8 @@ class BrowserUseAgent(Agent):
             # Log token usage summary
             await self.token_cost_service.log_usage_summary()
 
+            self.save_history(os.path.join(self.file_system_path, 'AgentHistory.json'))
+
             # Unregister signal handlers before cleanup
             signal_handler.unregister()
 
@@ -777,7 +693,7 @@ class BrowserUseAgent(Agent):
         else:
             # Exact matching
             return action_type == allowed_pattern
-    
+
     def _is_action_parallel_allowed(self, action: ActionModel) -> bool:
         """
         Check if an action is allowed to be executed in parallel.
@@ -790,16 +706,16 @@ class BrowserUseAgent(Agent):
         """
         action_data = action.model_dump(exclude_unset=True)
         action_type = next(iter(action_data.keys())) if action_data else None
-        
+
         if not action_type:
             return False
-            
+
         for allowed_pattern in self.allow_parallel_action_types:
             if self._matches_action_type(action_type, allowed_pattern):
                 return True
-                
+
         return False
-    
+
     def _group_actions_for_parallel_execution(self, actions: list[ActionModel]) -> list[list[ActionModel]]:
         """
         Group consecutive actions that can be executed in parallel.
@@ -812,27 +728,27 @@ class BrowserUseAgent(Agent):
         """
         if not actions:
             return []
-            
+
         groups = []
         current_group = [actions[0]]
-        
+
         for i in range(1, len(actions)):
             current_action = actions[i]
-            previous_action = actions[i-1]
-            
+            previous_action = actions[i - 1]
+
             # Check if both current and previous actions can be executed in parallel
             if (self._is_action_parallel_allowed(current_action) and
-                self._is_action_parallel_allowed(previous_action)):
+                    self._is_action_parallel_allowed(previous_action)):
                 # Add to current group
                 current_group.append(current_action)
             else:
                 # Start a new group
                 groups.append(current_group)
                 current_group = [current_action]
-        
+
         # Add the last group
         groups.append(current_group)
-        
+
         return groups
 
     @observe_debug(ignore_input=True, ignore_output=True)
@@ -865,21 +781,22 @@ class BrowserUseAgent(Agent):
 
         # Group actions for potential parallel execution
         action_groups = self._group_actions_for_parallel_execution(actions)
-        
+
         # Track global action index for logging and DOM checks
         global_action_index = 0
 
         for group_index, action_group in enumerate(action_groups):
             group_size = len(action_group)
-            
+
             # Check if this group can be executed in parallel
             can_execute_in_parallel = (
-                group_size > 1 and
-                all(self._is_action_parallel_allowed(action) for action in action_group)
+                    group_size > 1 and
+                    all(self._is_action_parallel_allowed(action) for action in action_group)
             )
-            
+
             if can_execute_in_parallel:
-                self.logger.info(f'🚀 Executing {group_size} actions in parallel: group {group_index + 1}/{len(action_groups)}')
+                self.logger.info(
+                    f'🚀 Executing {group_size} actions in parallel: group {group_index + 1}/{len(action_groups)}')
                 # Execute actions in parallel using asyncio.gather
                 parallel_results = await self._execute_actions_in_parallel(
                     action_group, global_action_index, total_actions,
@@ -887,7 +804,7 @@ class BrowserUseAgent(Agent):
                 )
                 results.extend(parallel_results)
                 global_action_index += group_size
-                
+
                 # Check if any result indicates completion or error
                 if any(result.is_done or result.error for result in parallel_results):
                     break
@@ -895,14 +812,14 @@ class BrowserUseAgent(Agent):
                 # Execute actions sequentially
                 for local_index, action in enumerate(action_group):
                     i = global_action_index + local_index
-                    
+
                     # Original sequential execution logic continues here...
-                    if i > 0:
-                        # ONLY ALLOW TO CALL `done` IF IT IS A SINGLE ACTION
-                        if action.model_dump(exclude_unset=True).get('done') is not None:
-                            msg = f'Done action is allowed only as a single action - stopped after action {i} / {total_actions}.'
-                            self.logger.debug(msg)
-                            break
+                    # if i > 0:
+                    #     # ONLY ALLOW TO CALL `done` IF IT IS A SINGLE ACTION
+                    #     if action.model_dump(exclude_unset=True).get('done') is not None:
+                    #         msg = f'Done action is allowed only as a single action - stopped after action {i} / {total_actions}.'
+                    #         self.logger.debug(msg)
+                    #         break
 
                     # DOM synchronization check - verify element indexes are still valid AFTER first action
                     if action.get_index() is not None and i != 0:
@@ -929,7 +846,7 @@ class BrowserUseAgent(Agent):
                     except Exception as e:
                         self.logger.error(f'❌ Executing action {i + 1} failed: {type(e).__name__}: {e}')
                         raise e
-                
+
                 global_action_index += len(action_group)
 
         return results
@@ -944,11 +861,11 @@ class BrowserUseAgent(Agent):
             check_for_new_elements: bool
     ) -> list[ActionResult]:
         """Execute a group of actions in parallel using asyncio.gather"""
-        
+
         async def execute_single_parallel_action(action: ActionModel, action_index: int) -> ActionResult:
             """Execute a single action for parallel execution"""
             await self._raise_if_stopped_or_paused()
-            
+
             # Get action info for logging
             action_data = action.model_dump(exclude_unset=True)
             action_name = next(iter(action_data.keys())) if action_data else 'unknown'
@@ -957,42 +874,41 @@ class BrowserUseAgent(Agent):
             ).replace('{', '').replace('}', '').replace("'", '').strip().strip(',')
             action_params = str(action_params)
             action_params = f'{action_params[:122]}...' if len(action_params) > 128 else action_params
-            
+
             time_start = time.time()
             blue = '\033[34m'
             reset = '\033[0m'
             self.logger.info(f'  🦾 {blue}[PARALLEL ACTION {action_index + 1}/{total_actions}]{reset} {action_params}')
-            
+
             # Execute the action
-            result = await self.controller.act(
+            result = await self.tools.act(
                 action=action,
                 browser_session=self.browser_session,
                 file_system=self.file_system,
                 page_extraction_llm=self.settings.page_extraction_llm,
                 sensitive_data=self.sensitive_data,
                 available_file_paths=self.available_file_paths,
-                context=self.context,
             )
-            
+
             time_end = time.time()
             time_elapsed = time_end - time_start
-            
+
             green = '\033[92m'
             self.logger.debug(
                 f'☑️ Parallel action {action_index + 1}/{total_actions}: {green}{action_params}{reset} in {time_elapsed:.2f}s'
             )
-            
+
             return result
-        
+
         # Create tasks for parallel execution
         tasks = [
             execute_single_parallel_action(action, start_index + i)
             for i, action in enumerate(actions)
         ]
-        
+
         # Execute all tasks in parallel
         parallel_results = await asyncio.gather(*tasks, return_exceptions=True)
-        
+
         # Process results and handle any exceptions
         processed_results = []
         for i, result in enumerate(parallel_results):
@@ -1002,7 +918,7 @@ class BrowserUseAgent(Agent):
                 raise result
             else:
                 processed_results.append(result)
-        
+
         return processed_results
 
     async def _check_dom_synchronization(
@@ -1060,13 +976,13 @@ class BrowserUseAgent(Agent):
                 include_in_memory=True,
                 long_term_memory=msg,
             )
-        
+
         return None
 
     async def _execute_single_action(self, action: ActionModel, action_index: int, total_actions: int) -> ActionResult:
         """Execute a single action in sequential mode"""
         await self._raise_if_stopped_or_paused()
-        
+
         # Get action name from the action model
         action_data = action.model_dump(exclude_unset=True)
         action_name = next(iter(action_data.keys())) if action_data else 'unknown'
@@ -1076,24 +992,23 @@ class BrowserUseAgent(Agent):
         # Ensure action_params is always a string before checking length
         action_params = str(action_params)
         action_params = f'{action_params[:122]}...' if len(action_params) > 128 else action_params
-        
+
         time_start = time.time()
-        
+
         red = '\033[91m'
         green = '\033[92m'
         blue = '\033[34m'
         reset = '\033[0m'
-        
+
         self.logger.info(f'  🦾 {blue}[ACTION {action_index + 1}/{total_actions}]{reset} {action_params}')
 
-        result = await self.controller.act(
+        result = await self.tools.act(
             action=action,
             browser_session=self.browser_session,
             file_system=self.file_system,
             page_extraction_llm=self.settings.page_extraction_llm,
             sensitive_data=self.sensitive_data,
             available_file_paths=self.available_file_paths,
-            context=self.context,
         )
 
         time_end = time.time()
@@ -1102,5 +1017,5 @@ class BrowserUseAgent(Agent):
         self.logger.debug(
             f'☑️ Executed action {action_index + 1}/{total_actions}: {green}{action_params}{reset} in {time_elapsed:.2f}s'
         )
-        
+
         return result
