@@ -15,7 +15,8 @@ from pathlib import Path
 
 # VibeSurf components
 from vibe_surf.agents.vibe_surf_agent import VibeSurfAgent
-from vibe_surf.controller.vibesurf_controller import VibeSurfController
+from vibe_surf.tools.browser_use_tools import BrowserUseTools
+from vibe_surf.tools.vibesurf_tools import VibeSurfTools
 from vibe_surf.browser.browser_manager import BrowserManager
 from browser_use.llm.base import BaseChatModel
 from browser_use.llm.openai.chat import ChatOpenAI
@@ -29,9 +30,10 @@ logger = logging.getLogger(__name__)
 # Global VibeSurf components
 vibesurf_agent: Optional[VibeSurfAgent] = None
 browser_manager: Optional[BrowserManager] = None
-controller: Optional[VibeSurfController] = None
+vibesurf_tools: Optional[VibeSurfTools] = None
 llm: Optional[BaseChatModel] = None
 db_manager: Optional['DatabaseManager'] = None
+current_llm_profile_name: Optional[str] = None
 
 # Environment variables
 workspace_dir: str = ""
@@ -50,10 +52,13 @@ active_task: Optional[Dict[str, Any]] = None
 
 def get_all_components():
     """Get all components as a dictionary"""
+    global vibesurf_agent, browser_manager, vibesurf_tools, llm, db_manager, current_llm_profile_name
+    global workspace_dir, browser_execution_path, browser_user_data, active_mcp_server, envs
+
     return {
         "vibesurf_agent": vibesurf_agent,
         "browser_manager": browser_manager,
-        "controller": controller,
+        "tools": vibesurf_tools,
         "llm": llm,
         "db_manager": db_manager,
         "workspace_dir": workspace_dir,
@@ -61,21 +66,22 @@ def get_all_components():
         "browser_user_data": browser_user_data,
         "active_mcp_server": active_mcp_server,
         "active_task": active_task,
+        "current_llm_profile_name": current_llm_profile_name,
         "envs": envs
     }
 
 
 def set_components(**kwargs):
     """Update global components"""
-    global vibesurf_agent, browser_manager, controller, llm, db_manager
+    global vibesurf_agent, browser_manager, vibesurf_tools, llm, db_manager, current_llm_profile_name
     global workspace_dir, browser_execution_path, browser_user_data, active_mcp_server, envs
 
     if "vibesurf_agent" in kwargs:
         vibesurf_agent = kwargs["vibesurf_agent"]
     if "browser_manager" in kwargs:
         browser_manager = kwargs["browser_manager"]
-    if "controller" in kwargs:
-        controller = kwargs["controller"]
+    if "tools" in kwargs:
+        vibesurf_tools = kwargs["tools"]
     if "llm" in kwargs:
         llm = kwargs["llm"]
     if "db_manager" in kwargs:
@@ -90,6 +96,8 @@ def set_components(**kwargs):
         active_mcp_server = kwargs["active_mcp_server"]
     if "envs" in kwargs:
         envs = kwargs["envs"]
+    if "current_llm_profile_name" in kwargs:
+        envs = kwargs["current_llm_profile_name"]
 
 
 async def execute_task_background(
@@ -98,12 +106,15 @@ async def execute_task_background(
         task: str,
         llm_profile_name: str,
         upload_files: Optional[List[str]] = None,
+        agent_mode: str = "thinking",
         db_session=None
 ):
     """Background task execution function for single task with LLM profile support"""
-    global vibesurf_agent, active_task
+    global vibesurf_agent, active_task, current_llm_profile_name
 
     try:
+        current_llm_profile_name = llm_profile_name
+
         # Check if MCP server configuration needs update
         await _check_and_update_mcp_servers(db_session)
 
@@ -131,7 +142,8 @@ async def execute_task_background(
         result = await vibesurf_agent.run(
             task=task,
             upload_files=upload_files,
-            session_id=session_id
+            session_id=session_id,
+            agent_mode=agent_mode
         )
 
         # Update task status to completed
@@ -162,7 +174,7 @@ async def execute_task_background(
                     db_session,
                     task_id=task_id,
                     task_result=result,
-                    task_status=active_task.get("status", "completed"),
+                    task_status=active_task.get("status", "completed") if active_task else "completed",
                     report_path=report_path
                 )
                 await db_session.commit()
@@ -223,8 +235,8 @@ def clear_active_task():
 
 
 async def _check_and_update_mcp_servers(db_session):
-    """Check if MCP server configuration has changed and update controller if needed"""
-    global controller, active_mcp_server
+    """Check if MCP server configuration has changed and update tools if needed"""
+    global vibesurf_tools, active_mcp_server
 
     try:
         if not db_session:
@@ -238,21 +250,21 @@ async def _check_and_update_mcp_servers(db_session):
 
         # Compare with shared state
         if current_active_servers != active_mcp_server:
-            logger.info(f"MCP server configuration changed. Updating controller...")
+            logger.info(f"MCP server configuration changed. Updating tools...")
             logger.info(f"Old config: {active_mcp_server}")
             logger.info(f"New config: {current_active_servers}")
 
             # Update shared state
             active_mcp_server = current_active_servers.copy()
 
-            # Create new MCP server config for controller
+            # Create new MCP server config for tools
             mcp_server_config = await _build_mcp_server_config(active_profiles)
 
             # Unregister old MCP clients and register new ones
-            if controller:
-                await controller.unregister_mcp_clients()
-                controller.mcp_server_config = mcp_server_config
-                await controller.register_mcp_clients()
+            if vibesurf_tools:
+                await vibesurf_tools.unregister_mcp_clients()
+                vibesurf_tools.mcp_server_config = mcp_server_config
+                await vibesurf_tools.register_mcp_clients()
                 logger.info("✅ Controller MCP configuration updated successfully")
 
     except Exception as e:
@@ -277,7 +289,8 @@ async def _load_active_mcp_servers():
 
     try:
         if not db_manager:
-            return {}
+            logger.info("Database manager not available, returning empty MCP config")
+            return {"mcpServers": {}}
 
         from .database.queries import McpProfileQueries
 
@@ -298,36 +311,24 @@ async def _load_active_mcp_servers():
 
             except Exception as e:
                 logger.warning(f"Failed to load MCP servers from database: {e}")
-                return {}
+                return {"mcpServers": {}}
             finally:
                 break
 
     except Exception as e:
         logger.warning(f"Database not available for MCP server loading: {e}")
-        return {}
+        return {"mcpServers": {}}
 
 
 async def initialize_vibesurf_components():
     """Initialize VibeSurf components from environment variables and default LLM profile"""
-    global vibesurf_agent, browser_manager, controller, llm, db_manager
+    global vibesurf_agent, browser_manager, vibesurf_tools, llm, db_manager, current_llm_profile_name
     global workspace_dir, browser_execution_path, browser_user_data, envs
+    from vibe_surf import common
 
     try:
         # Load environment variables
-        env_workspace_dir = os.getenv("VIBESURF_WORKSPACE", "")
-        if not env_workspace_dir or not env_workspace_dir.strip():
-            # Set default workspace directory based on OS
-            if platform.system() == "Windows":
-                default_workspace = os.path.join(os.environ.get("APPDATA", ""), "VibeSurf")
-            elif platform.system() == "Darwin":  # macOS
-                default_workspace = os.path.join(os.path.expanduser("~"), "Library", "Application Support", "VibeSurf")
-            else:  # Linux and others
-                default_workspace = os.path.join(os.path.expanduser("~"), ".vibesurf")
-            workspace_dir = default_workspace
-        else:
-            workspace_dir = env_workspace_dir
-        workspace_dir = os.path.abspath(workspace_dir)
-        os.makedirs(workspace_dir, exist_ok=True)
+        workspace_dir = common.get_workspace_dir()
         logger.info("WorkSpace directory: {}".format(workspace_dir))
 
         # Load environment configuration from envs.json
@@ -352,13 +353,13 @@ async def initialize_vibesurf_components():
         browser_execution_path = os.getenv("BROWSER_EXECUTION_PATH", "")
         assert os.path.exists(browser_execution_path), "Please set the BROWSER_EXECUTION_PATH environment variable"
         browser_user_data = os.getenv("BROWSER_USER_DATA", "")
-        if not browser_user_data:
+        if not browser_user_data or not os.path.exists(browser_user_data):
             browser_user_data = os.path.join(workspace_dir, "browser_user_data",
                                              f"{os.path.basename(browser_execution_path)}-profile")
 
         # Get VibeSurf extension path
         vibesurf_extension = os.getenv("VIBESURF_EXTENSION", "")
-        if not vibesurf_extension.strip():
+        if not vibesurf_extension.strip() or not os.path.exists(vibesurf_extension):
             current_file = Path(__file__)
             project_root = current_file.parent.parent.absolute()
             vibesurf_extension = str(project_root / "chrome_extension")
@@ -401,8 +402,8 @@ async def initialize_vibesurf_components():
 
         db_manager = DatabaseManager(database_url)
 
-        # Initialize database tables
-        await db_manager.create_tables()
+        # Initialize database tables with migration support
+        await db_manager.create_tables(use_migrations=True)
         logger.info("✅ Database manager initialized successfully")
 
         # Initialize LLM from default profile (if available) or fallback to environment variables
@@ -421,6 +422,8 @@ async def initialize_vibesurf_components():
                 user_data_dir=browser_user_data,
                 headless=False,
                 keep_alive=True,
+                auto_download_pdfs=False,
+                highlight_elements=True,
                 custom_extensions=[envs["VIBESURF_EXTENSION"]],
                 window_size={"width": primary_monitor.width, "height": primary_monitor.height}
             )
@@ -435,19 +438,19 @@ async def initialize_vibesurf_components():
         # Load active MCP servers from database
         mcp_server_config = await _load_active_mcp_servers()
 
-        # Initialize vibesurf controller with MCP server config
-        controller = VibeSurfController(mcp_server_config=mcp_server_config)
+        # Initialize vibesurf tools with MCP server config
+        vibesurf_tools = VibeSurfTools(mcp_server_config=mcp_server_config)
 
         # Register MCP clients if there are any active MCP servers
         if mcp_server_config and mcp_server_config.get("mcpServers"):
-            await controller.register_mcp_clients()
+            await vibesurf_tools.register_mcp_clients()
             logger.info(f"✅ Registered {len(mcp_server_config['mcpServers'])} MCP servers")
 
         # Initialize VibeSurfAgent
         vibesurf_agent = VibeSurfAgent(
             llm=llm,
             browser_manager=browser_manager,
-            controller=controller,
+            tools=vibesurf_tools,
             workspace_dir=workspace_dir
         )
 
@@ -468,7 +471,7 @@ async def initialize_vibesurf_components():
 
 async def _initialize_default_llm():
     """Initialize LLM from default profile or fallback to environment variables"""
-    global db_manager
+    global db_manager, current_llm_profile_name
 
     try:
         # Try to get default LLM profile from database
@@ -487,6 +490,7 @@ async def _initialize_default_llm():
                         )
                         if profile_with_key:
                             llm_instance = create_llm_from_profile(profile_with_key)
+                            current_llm_profile_name = default_profile.profile_name
                             logger.info(f"✅ LLM initialized from default profile: {default_profile.profile_name}")
                             return llm_instance
                     break
@@ -529,8 +533,9 @@ async def update_llm_from_profile(profile_name: str):
 
                 # Update global state
                 llm = new_llm
-                if vibesurf_agent:
-                    vibesurf_agent.llm = new_llm
+                if vibesurf_agent and vibesurf_agent.token_cost_service:
+                    # FIX: Register new LLM with token cost service to maintain tracking
+                    vibesurf_agent.llm = vibesurf_agent.token_cost_service.register_llm(new_llm)
 
                 logger.info(f"✅ LLM updated to profile: {profile_name}")
                 return True
