@@ -99,6 +99,12 @@ class VibeSurfAPIClient {
           if (error.status >= 400 && error.status < 500) {
             throw error; // Client errors shouldn't be retried
           }
+          
+          // Don't retry on LLM connection failures
+          if (error.data && error.data.error === 'llm_connection_failed') {
+            console.log('[API] LLM connection failed - skipping retry');
+            throw error;
+          }
         }
 
         // Don't retry on timeout for the last attempt
@@ -169,7 +175,8 @@ class VibeSurfAPIClient {
       task_description,
       llm_profile_name,
       upload_files_path,
-      mcp_server_config
+      mcp_server_config,
+      agent_mode
     } = taskData;
 
     return this.post('/tasks/submit', {
@@ -177,7 +184,8 @@ class VibeSurfAPIClient {
       task_description,
       llm_profile_name,
       upload_files_path,
-      mcp_server_config
+      mcp_server_config,
+      agent_mode
     });
   }
 
@@ -235,6 +243,10 @@ class VibeSurfAPIClient {
     return this.post('/tasks/stop', { reason });
   }
 
+  async addNewTask(newTask) {
+    return this.post('/tasks/add-new-task', { reason: newTask });
+  }
+
   // Activity APIs
   async getTaskInfo(taskId) {
     return this.get(`/activity/${taskId}`);
@@ -398,6 +410,67 @@ class VibeSurfAPIClient {
     return this.get(`/config/llm/providers/${encodeURIComponent(providerName)}/models`);
   }
 
+  // Voice Profile Management
+  async getVoiceProfiles(activeOnly = true, voiceModelType = null, limit = 50, offset = 0) {
+    const params = { active_only: activeOnly, limit, offset };
+    if (voiceModelType) {
+      params.voice_model_type = voiceModelType;
+    }
+    return this.get('/voices/voice-profiles', { params });
+  }
+
+  async getVoiceProfile(profileName) {
+    return this.get(`/voices/${encodeURIComponent(profileName)}`);
+  }
+
+  async createVoiceProfile(profileData) {
+    return this.post('/voices/voice-profiles', profileData);
+  }
+
+  async updateVoiceProfile(profileName, updateData) {
+    return this.put(`/voices/voice-profiles/${encodeURIComponent(profileName)}`, updateData);
+  }
+
+  async deleteVoiceProfile(profileName) {
+    return this.delete(`/voices/voice-profiles/${encodeURIComponent(profileName)}`);
+  }
+
+  // Voice Models - matches the backend route @router.get("/models")
+  async getVoiceModels(modelType = null) {
+    let url = '/voices/models';
+    if (modelType) {
+      url += `?model_type=${encodeURIComponent(modelType)}`;
+    }
+    return this.get(url);
+  }
+
+  // Voice Recording API
+  async transcribeAudio(audioBlob, voiceProfileName = null) {
+    const formData = new FormData();
+    formData.append('audio_file', audioBlob, 'recording.webm');
+    
+    // Add voice profile name if provided
+    const params = {};
+    if (voiceProfileName) {
+      params.voice_profile_name = voiceProfileName;
+    }
+    
+    return this.post('/voices/asr', formData, {
+      params,
+      headers: {} // Let browser set Content-Type with boundary for FormData
+    });
+  }
+
+  // Get available ASR profiles
+  async getASRProfiles(activeOnly = true) {
+    return this.get('/voices/voice-profiles', {
+      params: {
+        voice_model_type: 'asr',
+        active_only: activeOnly
+      }
+    });
+  }
+
   // Environment Variables
   async getEnvironmentVariables() {
     return this.get('/config/environments');
@@ -416,6 +489,20 @@ class VibeSurfAPIClient {
     return this.put('/config/controller', configData);
   }
 
+  // Browser APIs
+  async getActiveBrowserTab() {
+    return this.get('/browser/active-tab');
+  }
+
+  async getAllBrowserTabs() {
+    return this.get('/browser/all-tabs');
+  }
+
+  // Agent APIs - Get available skills
+  async getAllSkills() {
+    return this.get('/agent/get_all_skills');
+  }
+
   // Utility methods
   delay(ms) {
     return new Promise(resolve => setTimeout(resolve, ms));
